<?php

namespace Drupal\Tests\usfedgov_google_analytics\Functional;

use Drupal\Core\Url;
use Drupal\Tests\BrowserTestBase;

/**
 * DAP JS query string parameters test.
 *
 * @group usfedgov_google_analytics
 */
class ParametersTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['node', 'usfedgov_google_analytics'];

  /**
   * {@inheritdoc}
   */
  protected $strictConfigSchema = FALSE;

  /**
   * The module configuration.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * The path to a node that is created for testing.
   *
   * @var string
   */
  protected $nodePath;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->config = \Drupal::configFactory()->getEditable('usfedgov_google_analytics.settings');

    // In the testing profile the front page is set to /user. So the DAP script
    // won't be attached to it. We must generate a page on which the script will
    // be attached.
    $this->drupalCreateContentType(['type' => 'page']);
    $this->nodePath = "node/" . $this->drupalCreateNode()->id();
  }

  /**
   * Test that configured settings are added to the attached JS URL.
   */
  public function testSettingsAttachment(): void {
    $script_html = '<script src="https://dap.digitalgov.gov/Universal-Federated-Analytics-Min.js?agency=test';

    $this->updateSetting('agency', 'test');
    $this->drupalGet($this->nodePath);
    $this->assertSession()->responseContains($script_html);

    $this->assertSession()->responseNotContains($script_html . '&amp;yt=true');
    $this->updateSetting('extra.yt', TRUE);
    $this->drupalGet($this->nodePath);
    $this->assertSession()->responseContains($script_html . '&amp;yt=true');
  }


  /**
   * Updates a module setting with a new value.
   *
   * This is mostly here because of the need to rebuild the library definitions
   * after each time that the settings are updated. That's necessary because
   * the query parameters are added to the libraries dynamically. If the
   * definitions aren't rebuilt, then they'll always contain the original query
   * string that was built on the first page load.
   *
   * @param string $setting_name
   *   The name of the setting inside of the
   *   usfedgov_google_analytics__settings key.
   * @param mixed $value
   *   The new value
   */
  protected function updateSetting(string $setting_name, $value) {
    $this->config->set('usfedgov_google_analytics__settings.' . $setting_name, $value)->save();
    \Drupal::service('library.discovery')->clearCachedDefinitions();
  }

}

